@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2015-2016 Raymond Hill
+    Copyright (C) 2015-2017 Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,38 +31,44 @@ if ( typeof vAPI !== 'object' || !vAPI.domFilterer ) {
     return;
 }
 
-var df = vAPI.domFilterer,
-    loggedSelectors = vAPI.loggedSelectors || {},
-    matchedSelectors = [],
-    selectors, i, selector;
+var loggedSelectors = vAPI.loggedSelectors || {},
+    matchedSelectors = [];
 
 
-// CSS selectors.
-selectors = df.jobQueue[2]._0.concat(df.jobQueue[3]._0);
-i = selectors.length;
-while ( i-- ) {
-    selector = selectors[i];
-    if ( loggedSelectors.hasOwnProperty(selector) ) {
-        continue;
+var evaluateSelector = function(selector) {
+    if (
+        loggedSelectors.hasOwnProperty(selector) === false &&
+        document.querySelector(selector) !== null
+    ) {
+        loggedSelectors[selector] = true;
+        matchedSelectors.push(selector);
     }
-    if ( document.querySelector(selector) === null ) {
-        continue;
-    }
-    loggedSelectors[selector] = true;
-    matchedSelectors.push(selector);
-}
-
-// Non-CSS selectors.
-var logHit = function(node, job) {
-    if ( !job.raw || loggedSelectors.hasOwnProperty(job.raw) ) {
-        return;
-    }
-    loggedSelectors[job.raw] = true;
-    matchedSelectors.push(job.raw);
 };
-for ( i = 4; i < df.jobQueue.length; i++ ) {
-    df.runJob(df.jobQueue[i], logHit);
-}
+
+// Simple CSS selector-based cosmetic filters.
+vAPI.domFilterer.simpleHideSelectors.entries.forEach(evaluateSelector);
+
+// Complex CSS selector-based cosmetic filters.
+vAPI.domFilterer.complexHideSelectors.entries.forEach(evaluateSelector);
+
+// Non-querySelector-able filters.
+vAPI.domFilterer.nqsSelectors.forEach(function(filter) {
+    if ( loggedSelectors.hasOwnProperty(filter) === false ) {
+        loggedSelectors[filter] = true;
+        matchedSelectors.push(filter);
+    }
+});
+
+// Procedural cosmetic filters.
+vAPI.domFilterer.proceduralSelectors.entries.forEach(function(pfilter) {
+    if (
+        loggedSelectors.hasOwnProperty(pfilter.raw) === false &&
+        pfilter.exec().length !== 0
+    ) {
+        loggedSelectors[pfilter.raw] = true;
+        matchedSelectors.push(pfilter.raw);
+    }
+});
 
 vAPI.loggedSelectors = loggedSelectors;
 
