@@ -686,10 +686,6 @@ var cosmeticFilterMapper = (function() {
         matchesFnName = 'webkitMatchesSelector';
     }
 
-    // Why the call to hideNode()?
-    //   Not all target nodes have necessarily been force-hidden,
-    //   do it now so that the inspector does not unhide these
-    //   nodes when disabling style tags.
     var nodesFromStyleTag = function(rootNode) {
         var filterMap = nodeToCosmeticFilterMap,
             selectors, selector,
@@ -697,7 +693,7 @@ var cosmeticFilterMapper = (function() {
             i, j;
 
         // CSS-based selectors: simple one.
-        selectors = vAPI.domFilterer.job2._0;
+        selectors = vAPI.domFilterer.simpleHideSelectors.entries;
         i = selectors.length;
         while ( i-- ) {
             selector = selectors[i];
@@ -713,9 +709,9 @@ var cosmeticFilterMapper = (function() {
                 }
             }
         }
-    
+
         // CSS-based selectors: complex one (must query from doc root).
-        selectors = vAPI.domFilterer.job3._0;
+        selectors = vAPI.domFilterer.complexHideSelectors.entries;
         i = selectors.length;
         while ( i-- ) {
             selector = selectors[i];
@@ -730,27 +726,16 @@ var cosmeticFilterMapper = (function() {
         }
 
         // Non-CSS selectors.
-        var runJobCallback = function(node, job) {
+        var runJobCallback = function(node, pfilter) {
             if ( filterMap.has(node) === false ) {
-                filterMap.set(node, job.raw);
+                filterMap.set(node, pfilter.raw);
             }
         };
-        for ( i = 4; i < vAPI.domFilterer.jobQueue.length; i++ ) {
-            vAPI.domFilterer.runJob(vAPI.domFilterer.jobQueue[i], runJobCallback);
-        }
+        vAPI.domFilterer.proceduralSelectors.forEachNode(runJobCallback);
     };
 
     var incremental = function(rootNode) {
-        var styleTags = vAPI.domFilterer.styleTags || [];
-        var styleTag;
-        var i = styleTags.length;
-        while ( i-- ) {
-            styleTag = styleTags[i];
-            if ( styleTag.sheet !== null ) {
-                styleTag.sheet.disabled = true;
-                styleTag[vAPI.sessionId] = true;
-            }
-        }
+        vAPI.domFilterer.userCSS.toggle(false);
         nodesFromStyleTag(rootNode);
     };
 
@@ -760,16 +745,7 @@ var cosmeticFilterMapper = (function() {
     };
 
     var shutdown = function() {
-        var styleTags = vAPI.domFilterer.styleTags || [];
-        var styleTag;
-        var i = styleTags.length;
-        while ( i-- ) {
-            styleTag = styleTags[i];
-            if ( styleTag.sheet !== null ) {
-                styleTag.sheet.disabled = false;
-                styleTag[vAPI.sessionId] = undefined;
-            }
-        }
+        vAPI.domFilterer.userCSS.toggle(true);
     };
 
     return {
@@ -834,6 +810,17 @@ var elementsFromSpecialSelector = function(selector) {
 
 /******************************************************************************/
 
+var getSvgRootChildren = function() {
+    if ( svgRoot.children ) {
+        return svgRoot.children;
+    } else {
+        var childNodes = Array.prototype.slice.apply(svgRoot.childNodes);
+        return childNodes.filter(function(node) {
+            return node.nodeType === Node.ELEMENT_NODE;
+        });
+    }
+};
+
 var highlightElements = function(scrollTo) {
     var wv = pickerRoot.contentWindow.innerWidth;
     var hv = pickerRoot.contentWindow.innerHeight;
@@ -842,6 +829,7 @@ var highlightElements = function(scrollTo) {
     var xl, xr, yt, yb, w, h, ws;
     var xlu = Number.MAX_VALUE, xru = 0, ytu = Number.MAX_VALUE, ybu = 0;
     var lists = highlightedElementLists;
+    var svgRootChildren = getSvgRootChildren();
 
     for ( var i = 0; i < lists.length; i++ ) {
         elems = lists[i];
@@ -881,7 +869,7 @@ var highlightElements = function(scrollTo) {
             if ( yt < ytu ) { ytu = yt; }
             if ( yb > ybu ) { ybu = yb; }
         }
-        svgRoot.children[i+1].setAttribute('d', islands.join('') || 'M0 0');
+        svgRootChildren[i+1].setAttribute('d', islands.join('') || 'M0 0');
     }
 
     svgRoot.firstElementChild.setAttribute('d', ocean.join(''));
