@@ -177,6 +177,14 @@ var safeQuerySelectorAll = function(node, selector) {
 
 /******************************************************************************/
 
+var rawFilterFromTextarea = function() {
+    var s = taCandidate.value,
+        pos = s.indexOf('\n');
+    return pos === -1 ? s.trim() : s.slice(0, pos).trim();
+};
+
+/******************************************************************************/
+
 var getElementBoundingClientRect = function(elem) {
     var rect = typeof elem.getBoundingClientRect === 'function' ?
         elem.getBoundingClientRect() :
@@ -481,13 +489,11 @@ var cosmeticFilterFromElement = function(elem) {
     }
 
     // Class(es)
-    if ( selector === '' ) {
-        v = elem.classList;
-        if ( v ) {
-            i = v.length || 0;
-            while ( i-- ) {
-                selector += '.' + CSS.escape(v.item(i));
-            }
+    v = elem.classList;
+    if ( v ) {
+        i = v.length || 0;
+        while ( i-- ) {
+            selector += '.' + CSS.escape(v.item(i));
         }
     }
 
@@ -635,7 +641,9 @@ var filtersFrom = function(x, y) {
     filterToDOMInterface.set
     @desc   Look-up all the HTML elements matching the filter passed in
             argument.
-    @param  string, a cosmetic of network filter.
+    @param  string, a cosmetic or network filter.
+    @param  function, called once all items matching the filter have been
+            collected.
     @return array, or undefined if the filter is invalid.
 
     filterToDOMInterface.preview
@@ -733,16 +741,15 @@ var filterToDOMInterface = (function() {
     // ways to compose a valid href to the same effective URL. One idea is to
     // normalize all a[href] on the page, but for now I will wait and see, as I
     // prefer to refrain from tampering with the page content if I can avoid it.
-    var fromCosmeticFilter = function(filter) {
+    var fromPlainCosmeticFilter = function(filter) {
         var elems;
         try {
             elems = document.querySelectorAll(filter);
         }
         catch (e) {
-            return fromProceduralCosmeticFilter(filter);
+            return;
         }
-        var out = [],
-            iElem = elems.length;
+        var out = [], iElem = elems.length;
         while ( iElem-- ) {
             out.push({ type: 'cosmetic', elem: elems[iElem]});
         }
@@ -751,108 +758,27 @@ var filterToDOMInterface = (function() {
 
     // https://github.com/gorhill/uBlock/issues/1772
     // Handle procedural cosmetic filters.
-    var fromProceduralCosmeticFilter = function(filter) {
-        if ( filter.charCodeAt(filter.length - 1) === 0x29 /* ')' */ ) {
-            var parts = reProceduralCosmeticFilter.exec(filter);
-            if (
-                parts !== null &&
-                proceduralCosmeticFilterFunctions.hasOwnProperty(parts[2])
-            ) {
-                return proceduralCosmeticFilterFunctions[parts[2]](
-                    parts[1].trim(),
-                    parts[3].trim()
-                );
-            }
+    var fromCompiledCosmeticFilter = function(raw) {
+        if ( typeof raw !== 'string' ) { return; }
+        var o;
+        try {
+            o = JSON.parse(raw);
+        } catch(ex) {
+            return;
         }
-    };
-
-    var reProceduralCosmeticFilter = /^(.*?):(matches-css|has|style|xpath)\((.+?)\)$/;
-
-    // Collection of handlers for procedural cosmetic filters.
-    var proceduralCosmeticFilterFunctions = {
-        'has': function(selector, arg) {
-            if ( selector === '' ) { return; }
-            var elems;
-            try {
-                elems = document.querySelectorAll(selector);
-                document.querySelector(arg);
-            } catch(ex) {
-                return;
-            }
-            var out = [], elem;
-            for ( var i = 0, n = elems.length; i < n; i++ ) {
-                elem = elems[i];
-                if ( elem.querySelector(arg) ) {
-                    out.push({ type: 'cosmetic', elem: elem });
-                }
-            }
-            return out;
-        },
-        'matches-css': function(selector, arg) {
-            if ( selector === '' ) { return; }
-            var elems;
-            try {
-                elems = document.querySelectorAll(selector);
-            } catch(ex) {
-                return;
-            }
-            var out = [], elem, style,
-                pos = arg.indexOf(':');
-            if ( pos === -1 ) { return; }
-            var prop = arg.slice(0, pos).trim(),
-                reText = arg.slice(pos + 1).trim();
-            if ( reText === '' ) { return; }
-            var re = reText !== '*' ?
-                new RegExp('^' + reText.replace(/[.+?${}()|[\]\\^]/g, '\\$&').replace(/\*+/g, '.*?') + '$') :
-                /./;
-            for ( var i = 0, n = elems.length; i < n; i++ ) {
-                elem = elems[i];
-                style = window.getComputedStyle(elem, null);
-                if ( re.test(style[prop]) ) {
-                    out.push({ type: 'cosmetic', elem: elem });
-                }
-            }
-            return out;
-        },
-        'style': function(selector, arg) {
-            if ( selector === '' || arg === '' ) { return; }
-            var elems;
-            try {
-                elems = document.querySelectorAll(selector);
-            } catch(ex) {
-                return;
-            }
-            var out = [];
-            for ( var i = 0, n = elems.length; i < n; i++ ) {
-                out.push({ type: 'cosmetic', elem: elems[i] });
-            }
-            lastAction = selector + ' { ' + arg + ' }';
-            return out;
-        },
-        'xpath': function(selector, arg) {
-            if ( selector !== '' ) { return []; }
-            var result;
-            try {
-                result = document.evaluate(
-                    arg,
-                    document,
-                    null,
-                    XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
-                    null
-                );
-            } catch(ex) {
-                return;
-            }
-            if ( result === undefined ) { return []; }
-            var out = [], elem, i = result.snapshotLength;
-            while ( i-- ) {
-                elem = result.snapshotItem(i);
-                if ( elem.nodeType === 1 ) {
-                    out.push({ type: 'cosmetic', elem: elem });
-                }
-            }
-            return out;
+        var elems;
+        if ( o.style ) {
+            elems = document.querySelectorAll(o.style[0]);
+            lastAction = o.style.join(' ');
+        } else if ( o.tasks ) {
+            elems = vAPI.domFilterer.createProceduralFilter(o).exec();
         }
+        if ( !elems ) { return; }
+        var out = [];
+        for ( var i = 0, n = elems.length; i < n; i++ ) {
+            out.push({ type: 'cosmetic', elem: elems[i] });
+        }
+        return out;
     };
 
     var lastFilter,
@@ -862,26 +788,44 @@ var filterToDOMInterface = (function() {
         applied = false,
         previewing = false;
 
-    var queryAll = function(filter) {
+    var queryAll = function(filter, callback) {
         filter = filter.trim();
         if ( filter === lastFilter ) {
-            return lastResultset;
+            callback(lastResultset);
+            return;
         }
         unapply();
         if ( filter === '' ) {
             lastFilter = '';
             lastResultset = [];
-        } else {
-            lastFilter = filter;
-            lastAction = undefined;
-            lastResultset = filter.lastIndexOf('##', 0) === 0 ?
-                fromCosmeticFilter(filter.slice(2)) :
-                fromNetworkFilter(filter);
-            if ( previewing ) {
-                apply(filter);
-            }
+            callback(lastResultset);
+            return;
         }
-        return lastResultset;
+        lastFilter = filter;
+        lastAction = undefined;
+        if ( filter.lastIndexOf('##', 0) === -1 ) {
+            lastResultset = fromNetworkFilter(filter);
+            if ( previewing ) { apply(); }
+            callback(lastResultset);
+            return;
+        }
+        var selector = filter.slice(2);
+        lastResultset = fromPlainCosmeticFilter(selector);
+        if ( lastResultset ) {
+            if ( previewing ) { apply(); }
+            callback(lastResultset);
+            return;
+        }
+        // Procedural cosmetic filter
+        vAPI.messaging.send(
+            'elementPicker',
+            { what: 'compileCosmeticFilterSelector', selector: selector },
+            function(response) {
+                lastResultset = fromCompiledCosmeticFilter(response);
+                if ( previewing ) { apply(); }
+                callback(lastResultset);
+            }
+        );
     };
 
     var applyHide = function() {
@@ -983,9 +927,11 @@ var filterToDOMInterface = (function() {
     var preview = function(filter) {
         previewing = filter !== false;
         if ( previewing ) {
-            if ( queryAll(filter) !== undefined ) {
-                apply();
-            }
+            queryAll(filter, function(items) {
+                if ( items !== undefined ) {
+                    apply();
+                }
+            });
         } else {
             unapply();
         }
@@ -999,70 +945,75 @@ var filterToDOMInterface = (function() {
     };
 })();
 
-// https://www.youtube.com/watch?v=nuUXJ6RfIik
-
 /******************************************************************************/
 
-var userFilterFromCandidate = function() {
-    var v = taCandidate.value;
-    var items = filterToDOMInterface.set(v);
-    if ( !items || items.length === 0 ) {
-        return false;
-    }
-
-    // https://github.com/gorhill/uBlock/issues/738
-    // Trim dots.
-    var hostname = window.location.hostname;
-    if ( hostname.slice(-1) === '.' ) {
-        hostname = hostname.slice(0, -1);
-    }
-
-    // Cosmetic filter?
-    if ( v.lastIndexOf('##', 0) === 0 ) {
-        return hostname + v;
-    }
-
-    // Assume net filter
-    var opts = [];
-
-    // If no domain included in filter, we need domain option
-    if ( v.lastIndexOf('||', 0) === -1 ) {
-        opts.push('domain=' + hostname);
-    }
-
-    var item = items[0];
-    if ( item.opts ) {
-        opts.push(item.opts);
-    }
-
-    if ( opts.length ) {
-        v += '$' + opts.join(',');
-    }
-
-    return v;
-};
-
-/******************************************************************************/
-
-var onCandidateChanged = function() {
-    var elems = [],
-        items = filterToDOMInterface.set(taCandidate.value),
-        valid = items !== undefined;
-    if ( valid ) {
-        for ( var i = 0; i < items.length; i++ ) {
-            elems.push(items[i].elem);
+var userFilterFromCandidate = function(callback) {
+    var v = rawFilterFromTextarea();
+    filterToDOMInterface.set(v, function(items) {
+        if ( !items || items.length === 0 ) {
+            callback();
+            return;
         }
-    }
-    pickerBody.querySelector('body section textarea + div').textContent = valid ?
-        items.length.toLocaleString() :
-        '0';
-    taCandidate.classList.toggle('invalidFilter', !valid);
-    dialog.querySelector('#create').disabled = elems.length === 0;
-    highlightElements(elems, true);
+
+        // https://github.com/gorhill/uBlock/issues/738
+        // Trim dots.
+        var hostname = window.location.hostname;
+        if ( hostname.slice(-1) === '.' ) {
+            hostname = hostname.slice(0, -1);
+        }
+
+        // Cosmetic filter?
+        if ( v.lastIndexOf('##', 0) === 0 ) {
+            callback(hostname + v);
+            return;
+        }
+
+        // Assume net filter
+        var opts = [];
+
+        // If no domain included in filter, we need domain option
+        if ( v.lastIndexOf('||', 0) === -1 ) {
+            opts.push('domain=' + hostname);
+        }
+
+        var item = items[0];
+        if ( item.opts ) {
+            opts.push(item.opts);
+        }
+
+        if ( opts.length ) {
+            v += '$' + opts.join(',');
+        }
+
+        callback(v);
+    });
 };
 
 /******************************************************************************/
 
+var onCandidateChanged = (function() {
+    var process = function(items) {
+        var elems = [], valid = items !== undefined;
+        if ( valid ) {
+            for ( var i = 0; i < items.length; i++ ) {
+                elems.push(items[i].elem);
+            }
+        }
+        pickerBody.querySelector('#resultsetCount').textContent = valid ?
+            items.length.toLocaleString() :
+            'E';
+        dialog.querySelector('section').classList.toggle('invalidFilter', !valid);
+        dialog.querySelector('#create').disabled = elems.length === 0;
+        highlightElements(elems, true);
+    };
+
+    return function() {
+        filterToDOMInterface.set(rawFilterFromTextarea(), process);
+    };
+})();
+
+/******************************************************************************/
+
 var candidateFromFilterChoice = function(filterChoice) {
     var slot = filterChoice.slot;
     var filters = filterChoice.filters;
@@ -1083,13 +1034,22 @@ var candidateFromFilterChoice = function(filterChoice) {
     // - Do not compute exact path.
     // - Discard narrowing directives.
     if ( filterChoice.modifier ) {
-        return filter.replace(/:nth-of-type\(\d+\)/, '');
+        filter = filter.replace(/:nth-of-type\(\d+\)/, '');
+        // Remove the id if one or more classes exist.
+        if ( filter.charAt(2) === '#' && filter.indexOf('.') !== -1 ) {
+            filter = filter.replace(/#[^#.]+/, '');
+        }
+        return filter;
     }
 
     // Return path: the target element, then all siblings prepended
     var selector = '', joiner = '';
     for ( ; slot < filters.length; slot++ ) {
         filter = filters[slot];
+        // Remove all classes when an id exists.
+        if ( filter.charAt(2) === '#' ) {
+            filter = filter.replace(/\..+$/, '');
+        }
         selector = filter.slice(2) + joiner + selector;
         // Stop at any element with an id: these are unique in a web page
         if ( filter.lastIndexOf('###', 0) === 0 ) {
@@ -1101,6 +1061,15 @@ var candidateFromFilterChoice = function(filterChoice) {
         }
         joiner = ' > ';
     }
+
+    // https://github.com/gorhill/uBlock/issues/2519
+    if (
+        slot === filters.length &&
+        document.querySelectorAll(selector).length > 1
+    ) {
+        selector = 'body > ' + selector;
+    }
+
     return '##' + selector;
 };
 
@@ -1132,8 +1101,8 @@ var onDialogClicked = function(ev) {
         // We have to exit from preview mode: this guarantees matching elements
         // will be found for the candidate filter.
         filterToDOMInterface.preview(false);
-        var filter = userFilterFromCandidate();
-        if ( filter ) {
+        userFilterFromCandidate(function(filter) {
+            if ( !filter ) { return; }
             var d = new Date();
             vAPI.messaging.send(
                 'elementPicker',
@@ -1143,9 +1112,9 @@ var onDialogClicked = function(ev) {
                     pageDomain: window.location.hostname
                 }
             );
-            filterToDOMInterface.preview(taCandidate.value);
+            filterToDOMInterface.preview(rawFilterFromTextarea());
             stopPicker();
-        }
+        });
     }
 
     else if ( ev.target.id === 'pick' ) {
@@ -1161,7 +1130,7 @@ var onDialogClicked = function(ev) {
         if ( filterToDOMInterface.previewing() ) {
             filterToDOMInterface.preview(false);
         } else {
-            filterToDOMInterface.preview(taCandidate.value);
+            filterToDOMInterface.preview(rawFilterFromTextarea());
         }
         highlightElements(targetElements, true);
     }
@@ -1210,7 +1179,10 @@ var showDialog = function(options) {
     populate(netFilterCandidates, '#netFilters');
     populate(cosmeticFilterCandidates, '#cosmeticFilters');
 
-    dialog.querySelector('ul').style.display = netFilterCandidates.length || cosmeticFilterCandidates.length ? '' : 'none';
+    dialog.querySelector('ul').style.display =
+        netFilterCandidates.length || cosmeticFilterCandidates.length
+            ? ''
+            : 'none';
     dialog.querySelector('#create').disabled = true;
 
     // Auto-select a candidate filter
@@ -1232,21 +1204,46 @@ var showDialog = function(options) {
 
 /******************************************************************************/
 
-var elementFromPoint = function(x, y) {
-    if ( !pickerRoot ) {
-        return null;
+var zap = function() {
+    if ( targetElements.length === 0 ) { return; }
+    var elem = targetElements[0],
+        style = window.getComputedStyle(elem);
+    // Heuristic to detect scroll-locking: remove such lock when detected.
+    if ( parseInt(style.zIndex, 10) >= 1000 || style.position === 'fixed' ) {
+        document.body.style.setProperty('overflow', 'auto', 'important');
+        document.documentElement.style.setProperty('overflow', 'auto', 'important');
     }
-    pickerRoot.style.pointerEvents = 'none';
-    var elem = document.elementFromPoint(x, y);
-    if ( elem === document.body || elem === document.documentElement ) {
-        elem = null;
-    }
-    pickerRoot.style.pointerEvents = '';
-    return elem;
+    elem.parentNode.removeChild(elem);
+    elem = elementFromPoint();
+    highlightElements(elem ? [elem] : []);
 };
 
 /******************************************************************************/
 
+var elementFromPoint = (function() {
+    var lastX, lastY;
+
+    return function(x, y) {
+        if ( x !== undefined ) {
+            lastX = x; lastY = y;
+        } else if ( lastX !== undefined ) {
+            x = lastX; y = lastY;
+        } else {
+            return null;
+        }
+        if ( !pickerRoot ) { return null; }
+        pickerRoot.style.pointerEvents = 'none';
+        var elem = document.elementFromPoint(x, y);
+        if ( elem === document.body || elem === document.documentElement ) {
+            elem = null;
+        }
+        pickerRoot.style.pointerEvents = '';
+        return elem;
+    };
+})();
+
+/******************************************************************************/
+
 var onSvgHovered = (function() {
     var timer = null;
     var mx = 0, my = 0;
@@ -1257,20 +1254,69 @@ var onSvgHovered = (function() {
         highlightElements(elem ? [elem] : []);
     };
 
-    var onMove = function(ev) {
+    return function onMove(ev) {
         mx = ev.clientX;
         my = ev.clientY;
         if ( timer === null ) {
             timer = vAPI.setTimeout(onTimer, 40);
         }
     };
+})();
 
-    return onMove;
+/******************************************************************************/
+
+var onSvgTouchStartStop = (function() {
+    var startX,
+        startY;
+    return function onTouch(ev) {
+        ev.preventDefault();
+        if ( ev.type === 'touchstart' ) {
+            startX = ev.touches[0].pageX;
+            startY = ev.touches[0].pageY;
+            return;
+        }
+        if ( startX === undefined ) { return; }
+        var stopX = ev.changedTouches[0].pageX,
+            stopY = ev.changedTouches[0].pageY,
+            distance = Math.sqrt(
+                Math.pow(stopX - startX, 2),
+                Math.pow(stopY - startY, 2)
+            );
+        // Swipe = exit element zapper/picker.
+        if ( distance > 32 ) {
+            stopPicker();
+            return;
+        }
+        // Interpret touch event as a click.
+        onSvgClicked({
+            type: 'click',
+            target: ev.target,
+            clientX: startX,
+            clientY: startY
+        });
+    };
 })();
 
 /******************************************************************************/
 
 var onSvgClicked = function(ev) {
+    // If zap mode, highlight element under mouse, this makes the zapper usable
+    // on touch screens.
+    if ( pickerBody.classList.contains('zap') ) {
+        var elem = targetElements.lenght !== 0 && targetElements[0];
+        if ( !elem || ev.target !== svgIslands ) {
+            elem = elementFromPoint(ev.clientX, ev.clientY);
+            if ( elem !== null ) {
+                highlightElements([elem]);
+                return;
+            }
+        }
+        zap();
+        if ( !ev.shiftKey ) {
+            stopPicker();
+        }
+        return;
+    }
     // https://github.com/chrisaljoudi/uBlock/issues/810#issuecomment-74600694
     // Unpause picker if:
     // - click outside dialog AND
@@ -1291,16 +1337,26 @@ var onSvgClicked = function(ev) {
 
 var svgListening = function(on) {
     var action = (on ? 'add' : 'remove') + 'EventListener';
-    svgRoot[action]('mousemove', onSvgHovered);
+    svgRoot[action]('mousemove', onSvgHovered, { passive: true });
 };
 
 /******************************************************************************/
 
 var onKeyPressed = function(ev) {
-    if ( ev.which === 27 ) {
+    // Delete
+    if ( ev.key === 'Delete' && pickerBody.classList.contains('zap') ) {
         ev.stopPropagation();
         ev.preventDefault();
+        zap();
+        return;
+    }
+    // Esc
+    if ( ev.key === 'Escape' || ev.which === 27 ) {
+        ev.stopPropagation();
+        ev.preventDefault();
+        filterToDOMInterface.preview(false);
         stopPicker();
+        return;
     }
 };
 
@@ -1356,6 +1412,8 @@ var stopPicker = function() {
     dialog.removeEventListener('click', onDialogClicked);
     svgListening(false);
     svgRoot.removeEventListener('click', onSvgClicked);
+    svgRoot.removeEventListener('touchstart', onSvgTouchStartStop);
+    svgRoot.removeEventListener('touchend', onSvgTouchStartStop);
     pickerStyle.parentNode.removeChild(pickerStyle);
     pickerRoot.parentNode.removeChild(pickerRoot);
     pickerRoot.onload = null;
@@ -1390,6 +1448,7 @@ var startPicker = function(details) {
 
     pickerBody = frameDoc.body;
     pickerBody.setAttribute('lang', navigator.language);
+    pickerBody.classList.toggle('zap', details.zap === true);
 
     dialog = pickerBody.querySelector('aside');
     dialog.addEventListener('click', onDialogClicked);
@@ -1401,6 +1460,8 @@ var startPicker = function(details) {
     svgOcean = svgRoot.firstChild;
     svgIslands = svgRoot.lastChild;
     svgRoot.addEventListener('click', onSvgClicked);
+    svgRoot.addEventListener('touchstart', onSvgTouchStartStop);
+    svgRoot.addEventListener('touchend', onSvgTouchStartStop);
     svgListening(true);
 
     window.addEventListener('scroll', onScrolled, true);
