@@ -19,6 +19,8 @@
     Home: https://github.com/gorhill/uBlock
 */
 
+// https://github.com/gorhill/uBlock/issues/2720
+
 /******************************************************************************/
 /******************************************************************************/
 
@@ -174,10 +176,9 @@ housekeep itself.
 
     vAPI.tabs.onPopupCreated = function(targetTabId, openerTabId) {
         var popup = popupCandidates[targetTabId];
-        if ( popup !== undefined ) {
-            return;
+        if ( popup === undefined ) {
+            popupCandidates[targetTabId] = new PopupCandidate(targetTabId, openerTabId);
         }
-        popupCandidates[targetTabId] = new PopupCandidate(targetTabId, openerTabId);
         popupCandidateTest(targetTabId);
     };
 
@@ -526,7 +527,8 @@ vAPI.tabs.onClosed = function(tabId) {
 vAPI.tabs.onPopupUpdated = (function() {
     // The same context object will be reused everytime. This also allows to
     // remember whether a popup or popunder was matched.
-    var context = {};
+    var context = {},
+        logData;
 
     // https://github.com/gorhill/uBlock/commit/1d448b85b2931412508aa01bf899e0b6f0033626#commitcomment-14944764
     // See if two URLs are different, disregarding scheme -- because the scheme
@@ -550,8 +552,9 @@ vAPI.tabs.onPopupUpdated = (function() {
     };
 
     var popupMatch = function(openerURL, targetURL, clickedURL, popupType) {
-        var openerHostname = µb.URI.hostnameFromURI(openerURL);
-        var openerDomain = µb.URI.domainFromHostname(openerHostname);
+        var openerHostname = µb.URI.hostnameFromURI(openerURL),
+            openerDomain = µb.URI.domainFromHostname(openerHostname),
+            result;
 
         context.pageHostname = openerHostname;
         context.pageDomain = openerDomain;
@@ -562,108 +565,110 @@ vAPI.tabs.onPopupUpdated = (function() {
         context.requestHostname = µb.URI.hostnameFromURI(targetURL);
         context.requestType = 'popup';
 
+        // https://github.com/gorhill/uBlock/issues/1735
+        //   Do not bail out on `data:` URI, they are commonly used for popups.
+        // https://github.com/uBlockOrigin/uAssets/issues/255
+        //   Do not bail out on `about:blank`: an `about:blank` popup can be
+        //   opened, with the sole purpose to serve as an intermediary in
+        //   a sequence of chained popups.
+        // https://github.com/uBlockOrigin/uAssets/issues/263#issuecomment-272615772
+        //   Do not bail out, period: the static filtering engine must be
+        //   able to examine all sorts of URLs for popup filtering purpose.
+
+        // Dynamic filtering makes sense only when we have a valid opener
+        // hostname.
         // https://github.com/gorhill/uBlock/commit/1d448b85b2931412508aa01bf899e0b6f0033626#commitcomment-14944764
         //   Ignore bad target URL. On Firefox, an `about:blank` tab may be
         //   opened for a new tab before it is filled in with the real target
         //   URL.
-        // https://github.com/gorhill/uBlock/issues/1735
-        //   Do not bail out on `data:` URI, they are commonly used for popups.
-        if (
-            context.requestHostname === '' &&
-            targetURL.startsWith('data:') === false
-        ) {
-            return '';
-        }
-
-        // Dynamic filtering makes sense only when we have a valid hostname.
-        if ( openerHostname !== '' ) {
-            // Check user switch first
-            if (
-                typeof clickedURL === 'string' &&
-                areDifferentURLs(targetURL, clickedURL) &&
-                µb.hnSwitches.evaluateZ('no-popups', openerHostname)
-            ) {
-                return 'ub:no-popups: ' + µb.hnSwitches.z + ' true';
+        if ( openerHostname !== '' && targetURL !== 'about:blank' ) {
+            // Check per-site switch first
+            if ( µb.hnSwitches.evaluateZ('no-popups', openerHostname) === true ) {
+                if (
+                    typeof clickedURL === 'string' &&
+                    areDifferentURLs(targetURL, clickedURL)
+                ) {
+                    logData = {
+                        source: 'switch',
+                        raw: 'no-popups: ' + µb.hnSwitches.z + ' true'
+                    };
+                    return 1;
+                }
             }
 
             // https://github.com/gorhill/uBlock/issues/581
             //   Take into account popup-specific rules in dynamic URL filtering, OR
             //   generic allow rules.
-            µb.sessionURLFiltering.evaluateZ(openerHostname, targetURL, popupType);
+            result = µb.sessionURLFiltering.evaluateZ(openerHostname, targetURL, popupType);
             if (
-                µb.sessionURLFiltering.r === 1 && µb.sessionURLFiltering.type === popupType ||
-                µb.sessionURLFiltering.r === 2
+                result === 1 && µb.sessionURLFiltering.type === popupType ||
+                result === 2
             ) {
-                return µb.sessionURLFiltering.toFilterString();
+                logData = µb.sessionURLFiltering.toLogData();
+                return result;
             }
 
             // https://github.com/gorhill/uBlock/issues/581
             //   Take into account `allow` rules in dynamic filtering: `block` rules
             //   are ignored, as block rules are not meant to block specific types
             //   like `popup` (just like with static filters).
-            µb.sessionFirewall.evaluateCellZY(openerHostname, context.requestHostname, popupType);
-            if ( µb.sessionFirewall.r === 2 ) {
-                return µb.sessionFirewall.toFilterString();
+            result = µb.sessionFirewall.evaluateCellZY(openerHostname, context.requestHostname, popupType);
+            if ( result === 2 ) {
+                logData = µb.sessionFirewall.toLogData();
+                return 2;
             }
         }
 
         // https://github.com/chrisaljoudi/uBlock/issues/323
         // https://github.com/chrisaljoudi/uBlock/issues/1142
         //   Don't block if uBlock is turned off in popup's context
-        var snfe = µb.staticNetFilteringEngine;
-        if (
-            µb.getNetFilteringSwitch(targetURL) &&
-            snfe.matchStringExactType(context, targetURL, popupType) !== undefined
-        ) {
-            return snfe.toResultString(µb.logger.isEnabled());
+        if ( µb.getNetFilteringSwitch(targetURL) ) {
+            result = µb.staticNetFilteringEngine.matchStringExactType(
+                context,
+                targetURL,
+                popupType
+            );
+            if ( result !== 0 ) {
+                logData = µb.staticNetFilteringEngine.toLogData();
+                return result;
+            }
         }
 
-        return '';
+        return 0;
     };
 
     var mapPopunderResult = function(popunderURL, popunderHostname, result) {
-        if ( result.startsWith('sb:') === false ) {
-            return '';
+        if (
+            logData === undefined ||
+            logData.source !== 'static' ||
+            logData.token === µb.staticNetFilteringEngine.noTokenHash
+        ) {
+            return 0;
         }
-        var snfe = µb.staticNetFilteringEngine;
-        var token = snfe.tokenRegister;
-        if ( token === '*' ) {
-            return '';
-        }
-        if ( token === '.' ) {
+        if ( logData.token === µb.staticNetFilteringEngine.dotTokenHash ) {
             return result;
         }
-        result = snfe.toResultString(true);
-        var re = snfe.filterRegexFromCompiled(result.slice(3));
-        if ( re === null ) {
-            return '';
-        }
-        var matches = re.exec(popunderURL);
-        if ( matches === null ) {
-            return '';
-        }
+        var re = new RegExp(logData.regex),
+            matches = re.exec(popunderURL);
+        if ( matches === null ) { return 0; }
         var beg = matches.index,
             end = beg + matches[0].length,
             pos = popunderURL.indexOf(popunderHostname);
-        if ( pos === -1 ) {
-            return '';
-        }
+        if ( pos === -1 ) { return 0; }
         // https://github.com/gorhill/uBlock/issues/1471
         //   We test whether the opener hostname as at least one character
         //   within matched portion of URL.
         // https://github.com/gorhill/uBlock/issues/1903
         //   Ignore filters which cause a match before the start of the
         //   hostname in the URL.
-        return beg >= pos &&
-               beg < pos + popunderHostname.length &&
-               end > pos ?
-           result :
-           '';
+        return beg >= pos && beg < pos + popunderHostname.length && end > pos
+            ? result
+            : 0;
     };
 
     var popunderMatch = function(openerURL, targetURL) {
         var result = popupMatch(targetURL, openerURL, null, 'popunder');
-        if ( µb.isBlockResult(result) ) {
+        if ( result === 1 ) {
             return result;
         }
         // https://github.com/gorhill/uBlock/issues/1010#issuecomment-186824878
@@ -672,24 +677,24 @@ vAPI.tabs.onPopupUpdated = (function() {
         //   a broad one, we will consider the opener tab to be a popunder tab.
         //   For now, a "broad" filter is one which does not touch any part of
         //   the hostname part of the opener URL.
-        var popunderURL = openerURL;
-        var popunderHostname = µb.URI.hostnameFromURI(popunderURL);
+        var popunderURL = openerURL,
+            popunderHostname = µb.URI.hostnameFromURI(popunderURL);
         if ( popunderHostname === '' ) {
-            return '';
+            return 0;
         }
         result = mapPopunderResult(
             popunderURL,
             popunderHostname,
             popupMatch(targetURL, popunderURL, null, 'popup')
         );
-        if ( result !== '' ) {
+        if ( result !== 0 ) {
             return result;
         }
         // https://github.com/gorhill/uBlock/issues/1598
         // Try to find a match against origin part of the opener URL.
         popunderURL = µb.URI.originFromURI(popunderURL);
         if ( popunderURL === '' ) {
-            return '';
+            return 0;
         }
         return mapPopunderResult(
             popunderURL,
@@ -732,32 +737,34 @@ vAPI.tabs.onPopupUpdated = (function() {
         }
 
         // Popup test.
-        var popupType = 'popup';
-        var result = popupMatch(openerURL, targetURL, µb.mouseURL, 'popup');
+        var popupType = 'popup',
+            result = popupMatch(openerURL, targetURL, µb.mouseURL, 'popup');
 
         // Popunder test.
-        if ( result === '' ) {
+        if ( result === 0 ) {
             result = popunderMatch(openerURL, targetURL);
-            if ( µb.isBlockResult(result) ) {
+            if ( result === 1 ) {
                 popupType = 'popunder';
             }
         }
 
         // Log only for when there was a hit against an actual filter (allow or block).
+        // https://github.com/gorhill/uBlock/issues/2776
         if ( µb.logger.isEnabled() ) {
             µb.logger.writeOne(
                 popupType === 'popup' ? openerTabId : targetTabId,
                 'net',
-                result,
+                result !== 0 ? logData : undefined,
                 popupType,
                 popupType === 'popup' ? targetURL : openerURL,
                 µb.URI.hostnameFromURI(context.rootURL),
                 µb.URI.hostnameFromURI(context.rootURL)
             );
         }
+        logData = undefined;
 
         // Not blocked
-        if ( µb.isAllowResult(result) ) {
+        if ( result !== 1 ) {
             return;
         }
 
