@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2016 Raymond Hill
+    Copyright (C) 2014-2017 Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -194,9 +194,7 @@ var matchBucket = function(url, hostname, bucket, start) {
 
 µBlock.whitelistFromString = function(s) {
     var whitelist = Object.create(null),
-        reInvalidHostname = /[^a-z0-9.\-\[\]:]/,
-        reHostnameExtractor = /([a-z0-9\[][a-z0-9.\-]*[a-z0-9\]])(?::[\d*]+)?\/(?:[^\x00-\x20\/]|$)[^\x00-\x20]*$/,
-        lines = s.split(/[\n\r]+/),
+        lineIter = new this.LineIterator(s),
         line, matches, key, directive, re;
 
     // Comment bucket must always be ready to be used.
@@ -205,8 +203,9 @@ var matchBucket = function(url, hostname, bucket, start) {
     // New set of directives, scrap cached data.
     directiveToRegexpMap.clear();
 
-    for ( var i = 0; i < lines.length; i++ ) {
-        line = lines[i].trim();
+    while ( !lineIter.eot() ) {
+        line = lineIter.next().trim();
+
         // https://github.com/gorhill/uBlock/issues/171
         // Skip empty lines
         if ( line === '' ) {
@@ -228,7 +227,7 @@ var matchBucket = function(url, hostname, bucket, start) {
             }
         }
         // Regex-based (ensure it is valid)
-        else if ( line.startsWith('/') && line.endsWith('/') ) {
+        else if ( line.length > 2 && line.startsWith('/') && line.endsWith('/') ) {
             key = '//';
             directive = line;
             try {
@@ -267,6 +266,28 @@ var matchBucket = function(url, hostname, bucket, start) {
     return whitelist;
 };
 
+µBlock.validateWhitelistString = function(s) {
+    var lineIter = new this.LineIterator(s), line;
+    while ( !lineIter.eot() ) {
+        line = lineIter.next().trim();
+        if ( line === '' ) { continue; }
+        if ( line.startsWith('#') ) { continue; } // Comment
+        if ( line.indexOf('/') === -1 ) { // Plain hostname
+            if ( reInvalidHostname.test(line) ) { return false; }
+            continue;
+        }
+        if ( line.length > 2 && line.startsWith('/') && line.endsWith('/') ) { // Regex-based
+            try { new RegExp(line.slice(1, -1)); } catch(ex) { return false; }
+            continue;
+        }
+        if ( reHostnameExtractor.test(line) === false ) { return false; } // URL
+    }
+    return true;
+};
+
+var reInvalidHostname = /[^a-z0-9.\-\[\]:]/,
+    reHostnameExtractor = /([a-z0-9\[][a-z0-9.\-]*[a-z0-9\]])(?::[\d*]+)?\/(?:[^\x00-\x20\/]|$)[^\x00-\x20]*$/;
+
 /******************************************************************************/
 
 })();
@@ -319,6 +340,9 @@ var matchBucket = function(url, hostname, bucket, start) {
             us.dynamicFilteringEnabled = true;
         }
         break;
+    case 'autoUpdate':
+        this.scheduleAssetUpdater(value ? 7 * 60 * 1000 : 0);
+        break;
     case 'collapseBlocked':
         if ( value === false ) {
             this.cosmeticFilteringEngine.removeFromSelectorCache('*', 'net');
@@ -368,11 +392,12 @@ var matchBucket = function(url, hostname, bucket, start) {
 
 /******************************************************************************/
 
-µBlock.elementPickerExec = function(tabId, targetElement) {
+µBlock.elementPickerExec = function(tabId, targetElement, zap) {
     if ( vAPI.isBehindTheSceneTabId(tabId) ) {
         return;
     }
     this.epickerTarget = targetElement || '';
+    this.epickerZap = zap || false;
     this.scriptlets.inject(tabId, 'element-picker');
     if ( typeof vAPI.tabs.select === 'function' ) {
         vAPI.tabs.select(tabId);
@@ -454,18 +479,6 @@ var matchBucket = function(url, hostname, bucket, start) {
 
 /******************************************************************************/
 
-µBlock.isBlockResult = function(result) {
-    return typeof result === 'string' && result.charCodeAt(1) === 98 /* 'b' */;
-};
-
-/******************************************************************************/
-
-µBlock.isAllowResult = function(result) {
-    return typeof result !== 'string' || result.charCodeAt(1) !== 98 /* 'b' */;
-};
-
-/******************************************************************************/
-
 µBlock.toggleHostnameSwitch = function(details) {
     if ( this.hnSwitches.toggleZ(details.name, details.hostname, !!details.deep, details.state) ) {
         this.saveHostnameSwitches();
