@@ -14,6 +14,7 @@ fi
 
 rm -rf $DES
 mkdir $DES
+cp    ./assets/assets.json                                       $DES/
 
 mkdir $DES/thirdparties
 cp -R ../uAssets/thirdparties/easylist-downloads.adblockplus.org $DES/thirdparties/
@@ -24,8 +25,5 @@ cp -R ../uAssets/thirdparties/www.malwaredomainlist.com          $DES/thirdparti
 
 mkdir $DES/ublock
 cp -R ../uAssets/filters/*                                       $DES/ublock/
-cp -R ./assets/ublock/filter-lists.json                          $DES/ublock/
-
-cp ../uAssets/checksums/ublock0.txt                              $DES/checksums.txt
 
 echo "done."
