@@ -2,7 +2,7 @@
 #
 # This script assumes a linux environment
 
-TEMPFILE=/tmp/uAsset
+TEMPFILE=(mktemp)
 
 echo "*** uAssets: updating remote assets..."
 
@@ -10,9 +10,9 @@ declare -A assets
 assets=(
     ['thirdparties/easylist-downloads.adblockplus.org/easylist.txt']='https://easylist.to/easylist/easylist.txt'
     ['thirdparties/easylist-downloads.adblockplus.org/easyprivacy.txt']='https://easylist.to/easylist/easyprivacy.txt'
-    ['thirdparties/mirror1.malwaredomains.com/files/justdomains']='http://mirror1.malwaredomains.com/files/justdomains'
+    ['thirdparties/mirror1.malwaredomains.com/files/justdomains']='https://mirror1.malwaredomains.com/files/justdomains'
     ['thirdparties/pgl.yoyo.org/as/serverlist']='https://pgl.yoyo.org/adservers/serverlist.php?hostformat=hosts&showintro=1&startdate%5Bday%5D=&startdate%5Bmonth%5D=&startdate%5Byear%5D=&mimetype=plaintext'
-    ['thirdparties/publicsuffix.org/list/effective_tld_names.dat']='https://publicsuffix.org/list/effective_tld_names.dat'
+    ['thirdparties/publicsuffix.org/list/effective_tld_names.dat']='https://publicsuffix.org/list/public_suffix_list.dat'
     ['thirdparties/www.malwaredomainlist.com/hostslist/hosts.txt']='https://www.malwaredomainlist.com/hostslist/hosts.txt'
 )
 
@@ -32,4 +32,6 @@ for i in "${!assets[@]}"; do
     fi
 done
 
-bash ./tools/update-ublock0.sh
+# This will help minimize diff between updates
+sort thirdparties/mirror1.malwaredomains.com/files/justdomains > $TEMPFILE
+mv $TEMPFILE thirdparties/mirror1.malwaredomains.com/files/justdomains
